@@ -7,6 +7,7 @@ from collections import ChainMap
 from typing import (
     Any,
     Callable,
+    Dict,
     Hashable,
     Iterable,
     List,
@@ -72,6 +73,7 @@ coordinate_criteria: MutableMapping[str, MutableMapping[str, Tuple]] = {
         "longitude": ("Lon",),
     },
     "axis": {"T": ("T",), "Z": ("Z",), "Y": ("Y",), "X": ("X",)},
+    "cartesian_axis": {"T": ("T",), "Z": ("Z",), "Y": ("Y",), "X": ("X",)},
     "positive": {"Z": ("up", "down"), "vertical": ("up", "down")},
     "units": {
         "latitude": (
@@ -91,17 +93,6 @@ coordinate_criteria: MutableMapping[str, MutableMapping[str, Tuple]] = {
             "degreesE",
         ),
     },
-    # "regular_expression": {
-    #     "time": r"time[0-9]*",
-    #     "vertical": (
-    #         r"(lv_|bottom_top|sigma|h(ei)?ght|altitude|depth|isobaric|pres|"
-    #         r"isotherm)[a-z_]*[0-9]*"
-    #     ),
-    #     "y": r"y",
-    #     "latitude": r"x?lat[a-z0-9]*",
-    #     "x": r"x",
-    #     "longitude": r"x?lon[a-z0-9]*",
-    # },
 }
 
 # "vertical" is just an alias for "Z"
@@ -111,6 +102,54 @@ coordinate_criteria["standard_name"]["vertical"] = coordinate_criteria["standard
 # "long_name" and "standard_name" criteria are the same. For convenience.
 coordinate_criteria["long_name"] = coordinate_criteria["standard_name"]
 
+
+#: regular expressions for guess_coord_axis
+regex = {
+    "time": "time[0-9]*",
+    "vertical": (
+        "(lv_|bottom_top|sigma|h(ei)?ght|altitude|depth|isobaric|pres|"
+        "isotherm)[a-z_]*[0-9]*"
+    ),
+    "Y": "y",
+    "latitude": "y?lat[a-z0-9]*",
+    "X": "x",
+    "longitude": "x?lon[a-z0-9]*",
+}
+regex["Z"] = regex["vertical"]
+regex["T"] = regex["time"]
+
+
+attrs = {
+    "X": {"axis": "X"},
+    "T": {"axis": "T", "standard_name": "time"},
+    "Y": {"axis": "Y"},
+    "Z": {"axis": "Z"},
+    "latitude": {"units": "degrees_north", "standard_name": "latitude"},
+    "longitude": {"units": "degrees_east", "standard_name": "longitude"},
+}
+attrs["time"] = attrs["T"]
+attrs["vertical"] = attrs["Z"]
+
+
+def _is_datetime_like(da: DataArray) -> bool:
+    import numpy as np
+
+    if np.issubdtype(da.dtype, np.datetime64) or np.issubdtype(
+        da.dtype, np.timedelta64
+    ):
+        return True
+
+    try:
+        import cftime
+
+        if isinstance(da.data[0], cftime.datetime):
+            return True
+    except ImportError:
+        pass
+
+    return False
+
+
 # Type for Mapper functions
 Mapper = Callable[[Union[DataArray, Dataset], str], List[str]]
 
@@ -175,7 +214,7 @@ def _get_axis_coord_single(var: Union[DataArray, Dataset], key: str,) -> List[st
     results = _get_axis_coord(var, key)
     if len(results) > 1:
         raise KeyError(
-            f"Multiple results for {key!r} found: {results!r}. Is this valid CF? Please open an issue."
+            f"Multiple results for {key!r} found: {results!r}. I expected only one."
         )
     elif len(results) == 0:
         raise KeyError(f"No results found for {key!r}.")
@@ -742,10 +781,11 @@ class CFAccessor:
                 maybe_update = {
                     # TODO: this is assuming key_mappers[k] is always
                     # _get_axis_coord_single
-                    k: apply_mapper(mapper, self._obj, v)[0]
+                    k: apply_mapper(
+                        key_mappers[k], self._obj, v, error=False, default=[v]
+                    )[0]
                     for k, v in kwargs[vkw].items()
                     if k in key_mappers
-                    for mapper in key_mappers[k]
                 }
                 kwargs[vkw].update(maybe_update)
 
@@ -857,42 +897,72 @@ class CFAccessor:
             ]
         )
 
-    def get_associated_variable_names(self, name: Hashable) -> List[Hashable]:
+    def get_associated_variable_names(self, name: Hashable) -> Dict[str, List[str]]:
         """
-        Returns a list of variable names referred to in the following attributes
-            1. "coordinates"
-            2. "cell_measures"
-            3. "ancillary_variables"
+        Returns a dict mapping
+            1. "ancillary_variables"
+            2. "bounds"
+            3. "cell_measures"
+            4. "coordinates"
+        to a list of variable names referred to in the appropriate attribute
+
+        Parameters
+        ----------
+
+        name: Hashable
+
+        Returns
+        ------
+
+        Dict with keys "ancillary_variables", "cell_measures", "coordinates", "bounds"
         """
-        coords = []
+        keys = ["ancillary_variables", "cell_measures", "coordinates", "bounds"]
+        coords: Dict[str, List[str]] = {k: [] for k in keys}
         attrs_or_encoding = ChainMap(self._obj[name].attrs, self._obj[name].encoding)
 
         if "coordinates" in attrs_or_encoding:
-            coords.extend(attrs_or_encoding["coordinates"].split(" "))
+            coords["coordinates"] = attrs_or_encoding["coordinates"].split(" ")
 
         if "cell_measures" in attrs_or_encoding:
-            measures = [
-                _get_measure(self._obj[name], measure)
-                for measure in _CELL_MEASURES
-                if measure in attrs_or_encoding["cell_measures"]
-            ]
-            coords.extend(*measures)
+            coords["cell_measures"] = list(
+                itertools.chain(
+                    *[
+                        _get_measure(self._obj[name], measure)
+                        for measure in _CELL_MEASURES
+                        if measure in attrs_or_encoding["cell_measures"]
+                    ]
+                )
+            )
 
         if (
             isinstance(self._obj, Dataset)
             and "ancillary_variables" in attrs_or_encoding
         ):
-            anames = attrs_or_encoding["ancillary_variables"].split(" ")
-            coords.extend(anames)
+            coords["ancillary_variables"] = attrs_or_encoding[
+                "ancillary_variables"
+            ].split(" ")
 
-        missing = set(coords) - set(self._maybe_to_dataset().variables)
+        if "bounds" in attrs_or_encoding:
+            coords["bounds"] = [attrs_or_encoding["bounds"]]
+
+        for dim in self._obj[name].dims:
+            dbounds = self._obj[dim].attrs.get("bounds", None)
+            if dbounds:
+                coords["bounds"].append(dbounds)
+
+        allvars = itertools.chain(*coords.values())
+        missing = set(allvars) - set(self._maybe_to_dataset().variables)
         if missing:
             warnings.warn(
                 f"Variables {missing!r} not found in object but are referred to in the CF attributes.",
                 UserWarning,
             )
-            for m in missing:
-                coords.remove(m)
+            for k, v in coords.items():
+                for m in missing:
+                    if m in v:
+                        v.remove(m)
+                        coords[k] = v
+
         return coords
 
     def __getitem__(self, key: Union[str, List[str]]):
@@ -916,7 +986,13 @@ class CFAccessor:
         successful = dict.fromkeys(key, False)
         for k in key:
             if k in _AXIS_NAMES + _COORD_NAMES:
-                names = axis_coord_mapper(self._obj, k)
+                try:
+                    names = axis_coord_mapper(self._obj, k)
+                except KeyError as e:
+                    raise KeyError(
+                        f"Receive multiple variables for key {k!r}. Expected only one. Please pass a list [{k!r}] instead to get all variables matching {k!r}."
+                    )
+                    raise e
                 successful[k] = bool(names)
                 coords.extend(names)
             elif k in _CELL_MEASURES:
@@ -936,8 +1012,12 @@ class CFAccessor:
         allnames = varnames + coords
 
         try:
-            for name in varnames:
-                coords.extend(self.get_associated_variable_names(name))
+            for name in allnames:
+                extravars = self.get_associated_variable_names(name)
+                # we cannot return bounds variables with scalar keys
+                if scalar_key:
+                    extravars.pop("bounds")
+                coords.extend(itertools.chain(*extravars.values()))
 
             if isinstance(self._obj, DataArray):
                 ds = self._obj._to_temp_dataset()
@@ -991,47 +1071,6 @@ class CFAccessor:
         else:
             return obj
 
-    def add_bounds(self, dims: Union[Hashable, Iterable[Hashable]]):
-        """
-        Returns a new object with bounds variables. The bounds values are guessed assuming
-        equal spacing on either side of a coordinate label.
-
-        Parameters
-        ----------
-        dims: Hashable or Iterable[Hashable]
-            Either a single dimension name or a list of dimension names.
-
-        Returns
-        -------
-        DataArray or Dataset with bounds variables added and appropriate "bounds" attribute set.
-
-        Notes
-        -----
-
-        The bounds variables are automatically named f"{dim}_bounds" where ``dim``
-        is a dimension name.
-        """
-        if isinstance(dims, Hashable):
-            dimensions = (dims,)
-        else:
-            dimensions = dims
-
-        bad_dims: Set[Hashable] = set(dimensions) - set(self._obj.dims)
-        if bad_dims:
-            raise ValueError(
-                f"{bad_dims!r} are not dimensions in the underlying object."
-            )
-
-        obj = self._maybe_to_dataset(self._obj.copy(deep=True))
-        for dim in dimensions:
-            bname = f"{dim}_bounds"
-            if bname in obj.variables:
-                raise ValueError(f"Bounds variable name {bname!r} will conflict!")
-            obj.coords[bname] = _guess_bounds_dim(obj[dim].reset_coords(drop=True))
-            obj[dim].attrs["bounds"] = bname
-
-        return self._maybe_to_dataarray(obj)
-
     def rename_like(
         self, other: Union[DataArray, Dataset]
     ) -> Union[DataArray, Dataset]:
@@ -1082,10 +1121,108 @@ class CFAccessor:
                 variable.attrs["coordinates"] = coordinates
         return self._maybe_to_dataarray(ds)
 
+    def guess_coord_axis(self, verbose: bool = False) -> Union[DataArray, Dataset]:
+        """
+        Automagically guesses X, Y, Z, T, latitude, longitude, and adds
+        appropriate attributes. Uses regexes from Metpy and inspired by Iris
+        function of same name.
+
+        Existing attributes will not be modified.
+
+        Parameters
+        ----------
+        verbose: bool
+            Print extra info to screen
+
+        Returns
+        -------
+        DataArray or Dataset with appropriate attributes added
+        """
+        import re
+
+        obj = self._obj.copy(deep=True)
+        for dim in obj.dims:
+            if _is_datetime_like(obj[dim]):
+                if verbose:
+                    print(
+                        f"I think {dim!r} is of type 'time' since it has a datetime-like type."
+                    )
+                obj[dim].attrs = dict(ChainMap(obj[dim].attrs, attrs["time"]))
+                continue  # prevent second detection
+
+            for axis, pattern in regex.items():
+                # match variable names
+                if re.match(pattern, dim.lower()):
+                    if verbose:
+                        print(
+                            f"I think {dim!r} is of type {axis!r} since it matched {pattern!r}"
+                        )
+                    obj[dim].attrs = dict(ChainMap(obj[dim].attrs, attrs[axis]))
+        return obj
+
 
 @xr.register_dataset_accessor("cf")
 class CFDatasetAccessor(CFAccessor):
-    pass
+    def get_bounds(self, key: str) -> DataArray:
+        """
+        Get bounds variable corresponding to key.
+
+        Parameters
+        ----------
+        key: str
+            Name of variable whose bounds are desired
+
+        Returns
+        -------
+        DataArray
+        """
+        name = apply_mapper(
+            _get_axis_coord_single, self._obj, key, error=False, default=[key]
+        )[0]
+        bounds = self._obj[name].attrs["bounds"]
+        obj = self._maybe_to_dataset()
+        return obj[bounds]
+
+    def add_bounds(self, dims: Union[Hashable, Iterable[Hashable]]):
+        """
+        Returns a new object with bounds variables. The bounds values are guessed assuming
+        equal spacing on either side of a coordinate label.
+
+        Parameters
+        ----------
+        dims: Hashable or Iterable[Hashable]
+            Either a single dimension name or a list of dimension names.
+
+        Returns
+        -------
+        DataArray or Dataset with bounds variables added and appropriate "bounds" attribute set.
+
+        Notes
+        -----
+
+        The bounds variables are automatically named f"{dim}_bounds" where ``dim``
+        is a dimension name.
+        """
+        if isinstance(dims, Hashable):
+            dimensions = (dims,)
+        else:
+            dimensions = dims
+
+        bad_dims: Set[Hashable] = set(dimensions) - set(self._obj.dims)
+        if bad_dims:
+            raise ValueError(
+                f"{bad_dims!r} are not dimensions in the underlying object."
+            )
+
+        obj = self._maybe_to_dataset(self._obj.copy(deep=True))
+        for dim in dimensions:
+            bname = f"{dim}_bounds"
+            if bname in obj.variables:
+                raise ValueError(f"Bounds variable name {bname!r} will conflict!")
+            obj.coords[bname] = _guess_bounds_dim(obj[dim].reset_coords(drop=True))
+            obj[dim].attrs["bounds"] = bname
+
+        return self._maybe_to_dataarray(obj)
 
 
 @xr.register_dataarray_accessor("cf")
